<?php
/**
 * @copyright: Copyright © 2021 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */
namespace Firebear\PlatformSap\Block\Adminhtml\Sales\Order\View\Info;

use Magento\Backend\Block\Template\Context;
use Magento\Sales\Api\OrderRepositoryInterface;

/**
 * Adminhtml order sap information block.
 *
 */
class OrderSapInfo extends \Magento\Backend\Block\Template
{
    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterface
     */
    protected $order;

    /**
     * OrderSapInfo constructor.
     * @param Context $context
     * @param OrderRepositoryInterface $orderRepository
     * @param array $data
     */
    public function __construct(
        Context $context,
        OrderRepositoryInterface $orderRepository,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->orderRepository = $orderRepository;
    }

    /**
     * Get order from request.
     *
     * @return \Magento\Sales\Api\Data\OrderInterface
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function getOrder()
    {
        if (!$this->order) {
            $orderId = $this->getRequest()->getParam('order_id');
            $this->order = $this->orderRepository->get($orderId);
        }

        return $this->order;
    }

    /**
     * Determines can block be displayed or not.
     *
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function canShow()
    {
        return $this->getSapId() !== null;
    }

    /**
     * Get order sbo docnum attribute.
     *
     * @return int|null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getSapId()
    {
        $sboDocnum = null;
        $order = $this->getOrder();
        if ($order->getData('sap_id') !== null) {
            $sboDocnum = $order->getData('sap_id');
        }

        return $sboDocnum;
    }
}
