<?php
/**
 * @copyright: Copyright © 2020 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Model\Export\Adapter;

use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\Filesystem;
use Magento\ImportExport\Model\Export\Adapter\Csv as AbstractAdapter;
use Magento\Sales\Model\OrderRepository;

/**
 * Csv Export Adapter
 */
class Order extends AbstractAdapter
{
    const DOCNUM_FIELD = 'sap_id';
    /**
     * Adapter Data
     *
     * @var []
     */
    protected $_data;

    /**
     * @var \Firebear\ImportExport\Model\Export\Adapter\Gateway\Order
     */
    protected $gateway;

    /**
     * Order data
     *
     * @var []
     */
    private $orderData;

    /**
     * @var OrderRepository
     */
    private $orderRepository;

    /**
     * Order constructor.
     * @param Filesystem $filesystem
     * @param Gateway\Order $gateway
     * @param OrderRepository $orderRepository
     * @param null $destination
     * @param string $destinationDirectoryCode
     * @param array $data
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function __construct(
        Filesystem $filesystem,
        \Firebear\PlatformSap\Model\Export\Adapter\Gateway\Order $gateway,
        OrderRepository $orderRepository,
        $destination = null,
        $destinationDirectoryCode = DirectoryList::VAR_DIR,
        array $data = []
    ) {
        $this->gateway = $gateway;
        $this->orderRepository = $orderRepository;
        $this->_data = $data;

        parent::__construct(
            $filesystem,
            $destination,
            $destinationDirectoryCode
        );
    }

    /**
     * Write row data to source file.
     *
     * @param array $rowData
     * @throws \Exception
     * @return $this
     */
    public function writeRow(array $rowData)
    {
        if ((isset($rowData['increment_id']))
            && (isset($rowData['entity_id']))
            && (!isset($this->orderData[$rowData['increment_id']]['exported']))) {
            $order = $this->orderRepository->get($rowData['entity_id']);
            $this->gateway->addOrder($order);
            $this->orderData[$rowData['increment_id']]['exported'] = true;
        }

        if (null === $this->_headerCols) {
            $this->setHeaderCols(array_keys($rowData));
        }
        if (null === $this->_headerCols) {
            $this->_headerCols = [];
        }
        $this->_fileHandler->writeCsv(
            array_merge(
                $this->_headerCols,
                array_intersect_key($rowData, $this->_headerCols)
            ),
            $this->_delimiter,
            $this->_enclosure
        );
        return $this;
    }
}
