<?php
 /**
  * Copyright © Firebear Studio, Inc. All rights reserved.
  * See COPYING.txt for license details.
  */
namespace Firebear\PlatformSap\Model\Import\Source;

use Firebear\PlatformSap\Model\Import\Source\Gateway\ProductTierPrice as ProductTierPriceGateway;
use Magento\Framework\App\RequestInterface;

/**
 * Sap product price source
 */
class ProductTierPrice extends AbstractSource
{
    /**
     * @var \Firebear\PlatformSap\Model\Source\Gateway\Product
     */
    protected $gateway;

    /**
     * Config data
     *
     * @var array
     */
    protected $_data = [];

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $request;

    /**
     * ProductPrice constructor.
     * @param ProductTierPriceGateway $gateway
     * @param RequestInterface $request
     * @param null $data
     * @throws \Exception
     */
    public function __construct(
        ProductTierPriceGateway $gateway,
        RequestInterface $request,
        $data = null
    ) {
        $this->gateway = $gateway;
        $this->request = $request;
        $this->parseEntities($data);
    }

    /**
     * @param $data
     * @return array|bool
     * @throws \Exception
     */
    protected function parseEntities($data)
    {
        $this->entities = parent::parseEntities($data);
        if (!empty($this->entities)) {
            foreach ($this->entities as $key => $entity) {
                if (isset($entity['ItemCode'])
                    && isset($entity['Amount'])
                    && isset($entity['Discount'])
                ) {
                    if (!empty($data['sap_tier_price_website'])) {
                        $website = $data['sap_tier_price_website'];
                    } else {
                        $website = 'All';
                    }

                    $this->entities[$key] = [
                        'sku' => $entity['ItemCode'],
                        'tier_price_website' => $website,
                        'tier_price_customer_group' => 'ALL GROUPS',
                        'tier_price_qty' => $entity['Amount'],
                        'tier_price' => $entity['Discount'],
                        'tier_price_value_type' => 'Discount'
                    ];
                } else {
                    unset($this->entities[$key]);
                }
            }
        }

        $this->entities = array_values($this->entities);
        $this->_colNames = array_keys($this->entities[0] ?? []);
        return $this->entities;
    }
}
