<?php
/**
 * Copyright © Firebear Studio GmbH, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace Firebear\PlatformSap\Model;

use Firebear\PlatformSap\Api\RefundOrderInterface;
use Magento\Sales\Model\OrderFactory;
use Magento\Sales\Api\Data\CreditmemoCommentCreationInterface;
use Magento\Sales\Api\Data\CreditmemoCreationArgumentsInterface;
use Firebear\PlatformSap\Model\Export\Adapter\Order;

/**
 * Class ShipOrder
 */
class RefundOrder implements RefundOrderInterface
{
    /**
     * @var \Magento\Sales\Api\RefundOrderInterface
     */
    protected $refundOrder;

    /**
     * @var OrderFactory
     */
    protected $orderFactory;

    /**
     * RefundOrder constructor.
     * @param \Magento\Sales\Api\RefundOrderInterface $refundOrder
     * @param OrderFactory $orderFactory
     */
    public function __construct(
        \Magento\Sales\Api\RefundOrderInterface $refundOrder,
        OrderFactory $orderFactory
    ) {
        $this->refundOrder = $refundOrder;
        $this->orderFactory = $orderFactory;
    }

    /**
     * @param int $orderId
     * @param array $items
     * @param false $notify
     * @param false $appendComment
     * @param CreditmemoCommentCreationInterface|null $comment
     * @param CreditmemoCreationArgumentsInterface|null $arguments
     * @return int
     */
    public function execute(
        $orderId,
        array $items = [],
        $notify = false,
        $appendComment = false,
        CreditmemoCommentCreationInterface $comment = null,
        CreditmemoCreationArgumentsInterface $arguments = null
    ) {
        $order = $this->orderFactory->create()->loadByAttribute(Order::DOCNUM_FIELD, $orderId);
        $orderId = $order->getId() ? $order->getId() : $orderId;

        foreach ($items as $item) {
            foreach ($order->getItems() as $orderItem) {
                if ($item->getSku() == $orderItem->getSku()) {
                    $item->setOrderItemId($orderItem->getItemId());
                }
            }
        }

        return $this->refundOrder->execute(
            $orderId,
            $items,
            $notify,
            $appendComment,
            $comment,
            $arguments
        );
    }
}
