<?php
/**
 * @copyright: Copyright © 2021 Firebear Studio. All rights reserved.
 * @author   : Firebear Studio <fbeardev@gmail.com>
 */

namespace Firebear\PlatformSap\Setup\Patch\Data;

use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Category;
use Magento\Catalog\Model\ResourceModel\Eav\Attribute as CatalogAttribute;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Framework\Setup\Patch\PatchRevertableInterface;

/**
 * Create custom product attributes
 */
class UpdateCatalogAttributes implements DataPatchInterface, PatchRevertableInterface
{
    const ATTRIBUTE_CODE = 'sap_id';
    const ATTRIBUTE_GROUP = 'General';

    /**
     * @var ModuleDataSetupInterface
     */
    private $moduleDataSetup;

    /**
     * @var EavSetupFactory
     */
    private $eavSetupFactory;

    /**
     * @var EavSetup
     */
    private $setup;

    /**
     * UpdateCatalogAttributes constructor.
     * @param ModuleDataSetupInterface $moduleDataSetup
     * @param EavSetupFactory $eavSetupFactory
     */
    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        EavSetupFactory $eavSetupFactory
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->eavSetupFactory = $eavSetupFactory;
    }

    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function apply()
    {
        $setup = $this->getSetup();

        $setup->addAttribute(
            Product::ENTITY,
            self::ATTRIBUTE_CODE,
            [
                'label' => __('Sap Id'),
                'visible_on_front' => 1,
                'required' => 0,
                'global' => CatalogAttribute::SCOPE_STORE,
                'group' => self::ATTRIBUTE_GROUP
            ]
        );

        $setup->addAttribute(
            Product::ENTITY,
            'unit_of_measure',
            [
                'label' => __('Unit of measure'),
                'visible_on_front' => 1,
                'required' => 0,
                'global' => CatalogAttribute::SCOPE_STORE,
                'group' => self::ATTRIBUTE_GROUP
            ]
        );

        $this->assignAttributeToSets('manufacturer');

        $setup->addAttribute(
            Category::ENTITY,
            self::ATTRIBUTE_CODE,
            [
                'type' => 'varchar',
                'label' => __('Sap ID'),
                'group' => 'General Information',
                'visible' => false,
                'required' => false,
                'user_defined' => false,
            ]
        );
    }

    /**
     * @return EavSetup
     */
    protected function getSetup()
    {
        if (!$this->setup) {
            /** @var EavSetup $eavSetup */
            $this->setup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);
        }
        return $this->setup;
    }

    /**
     * @return array|string[]
     */
    public static function getDependencies()
    {
        return [];
    }

    /**
     * @return array|string[]
     */
    public function getAliases()
    {
        return [];
    }

    /**
     * Assign product attributes to sets
     */
    protected function assignAttributeToSets($attributeCode)
    {
        $setup = $this->getSetup();
        $entityTypeId = $setup->getEntityTypeId(Product::ENTITY);
        $attributeSetIds = $setup->getAllAttributeSetIds($entityTypeId);

        foreach ($attributeSetIds as $attributeSetId) {
            $setup->addAttributeToGroup(
                Product::ENTITY,
                $attributeSetId,
                self::ATTRIBUTE_GROUP,
                $attributeCode
            );
        }
    }

    /**
     *  Remove custom catalog attributes
     */
    public function revert()
    {
        $setup = $this->getSetup();

        $setup->removeAttribute(
            Product::ENTITY,
            self::ATTRIBUTE_CODE
        );

        $setup->removeAttribute(
            Product::ENTITY,
            'unit_of_measure'
        );

        $setup->removeAttribute(
            Category::ENTITY,
            self::ATTRIBUTE_CODE
        );
    }
}
